@@ -260,6 +260,15 @@ static BreakOpportunity row_COMBINING_MARK[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
+static BreakOpportunity row_WORD_JOINER[INDEX_END_OF_TABLE] = {
+  BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
+  BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
+  BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES,
+  BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES,
+  BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED,
+  BREAK_PROHIBITED
+};
+
 static BreakOpportunity *line_break_rows[INDEX_END_OF_TABLE] = {
   row_OPEN_PUNCTUATION, /* INDEX_OPEN_PUNCTUATION */
   row_CLOSE_PUNCTUATION, /* INDEX_CLOSE_PUNCTUATION */
@@ -280,7 +289,8 @@ static BreakOpportunity *line_break_rows[INDEX_END_OF_TABLE] = {
   row_BEFORE, /* INDEX_BEFORE */
   row_BEFORE_AND_AFTER, /* INDEX_BEFORE_AND_AFTER */
   row_ZERO_WIDTH_SPACE, /* INDEX_ZERO_WIDTH_SPACE */
-  row_COMBINING_MARK /* INDEX_COMBINING_MARK */
+  row_COMBINING_MARK, /* INDEX_COMBINING_MARK */
+  row_WORD_JOINER /* INDEX_WORD_JOINER */
 };
 
 /* Map GUnicodeBreakType to table indexes */
@@ -360,7 +370,9 @@ static int line_break_indexes[] = {
 #define LATIN(wc) (((wc) >= 0x0020 && (wc) <= 0x02AF) || ((wc) >= 0x1E00 && (wc) <= 0x1EFF))
 #define CYRILLIC(wc) (((wc) >= 0x0400 && (wc) <= 0x052F))
 #define GREEK(wc) (((wc) >= 0x0370 && (wc) <= 0x3FF) || ((wc) >= 0x1F00 && (wc) <= 0x1FFF))
-#define BACKSPACE_DELETES_CHARACTER(wc) (!LATIN (wc) && !CYRILLIC (wc) && !GREEK (wc))
+#define KANA(wc) ((wc) >= 0x3040 && (wc) <= 0x30FF)
+#define HANGUL(wc) ((wc) >= 0xAC00 && (wc) <= 0xD7A3)
+#define BACKSPACE_DELETES_CHARACTER(wc) (!LATIN (wc) && !CYRILLIC (wc) && !GREEK (wc) && !KANA(wc) && !HANGUL(wc))
 
 
 /* p. 132-133 of Unicode spec table 5-6 will help understand this */
@@ -820,6 +832,7 @@ pango_default_break (const gchar   *text,
             case G_UNICODE_COMBINING_MARK:
             case G_UNICODE_ENCLOSING_MARK:
             case G_UNICODE_NON_SPACING_MARK:
+	    case G_UNICODE_FORMAT:
               /* nothing, we just eat these up as part of the word */
               break;
 
