@@ -40,16 +40,18 @@ struct _PangoAttrIterator
   int end_index;
 };
 
-static PangoAttribute *pango_attr_color_new  (const PangoAttrClass *klass,
-                                              guint16               red,
-                                              guint16               green,
-                                              guint16               blue);
-static PangoAttribute *pango_attr_string_new (const PangoAttrClass *klass,
-                                              const char           *str);
-static PangoAttribute *pango_attr_int_new    (const PangoAttrClass *klass,
-                                              int                   value);
-static PangoAttribute *pango_attr_float_new  (const PangoAttrClass *klass,
-                                              double                value);
+static PangoAttribute *pango_attr_color_new         (const PangoAttrClass *klass,
+                                                     guint16               red,
+                                                     guint16               green,
+                                                     guint16               blue);
+static PangoAttribute *pango_attr_string_new        (const PangoAttrClass *klass,
+                                                     const char           *str);
+static PangoAttribute *pango_attr_int_new           (const PangoAttrClass *klass,
+                                                     int                   value);
+static PangoAttribute *pango_attr_float_new         (const PangoAttrClass *klass,
+                                                     double                value);
+static PangoAttribute *pango_attr_size_new_internal (int                   size,
+						     gboolean              absolute);
 
 
 /**
@@ -396,25 +398,79 @@ pango_attr_float_new  (const PangoAttrClass *klass,
   return (PangoAttribute *)result;
 }
 
+static PangoAttribute *
+pango_attr_size_copy (const PangoAttribute *attr)
+{
+  const PangoAttrSize *size_attr = (PangoAttrSize *)attr;
+  
+  return pango_attr_size_new_internal (size_attr->size, size_attr->absolute);
+}
+
+static void
+pango_attr_size_destroy (PangoAttribute *attr)
+{
+  g_free (attr);
+}
+
+static gboolean
+pango_attr_size_equal (const PangoAttribute *attr1,
+		       const PangoAttribute *attr2)
+{
+  const PangoAttrSize *size_attr1 = (const PangoAttrSize *)attr1;
+  const PangoAttrSize *size_attr2 = (const PangoAttrSize *)attr2;
+  
+  return (size_attr1->size == size_attr2->size &&
+	  size_attr1->absolute == size_attr2->absolute);
+}
+
+static PangoAttribute *
+pango_attr_size_new_internal (int size,
+			      gboolean absolute)
+{
+  PangoAttrSize *result;
+  static const PangoAttrClass klass = {
+    PANGO_ATTR_SIZE,
+    pango_attr_size_copy,
+    pango_attr_size_destroy,
+    pango_attr_size_equal
+  };
+
+  result = g_new (PangoAttrSize, 1);
+  result->attr.klass = &klass;
+  result->size = size;
+  result->absolute = absolute;
+
+  return (PangoAttribute *)result;
+}
+
 /**
  * pango_attr_size_new:
  * @size: the font size, in #PANGO_SCALE<!-- -->ths of a point.
  * 
- * Create a new font-size attribute.
+ * Create a new font-size attribute in fractional points.
  * 
  * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_size_new (int size)
 {
-  static const PangoAttrClass klass = {
-    PANGO_ATTR_SIZE,
-    pango_attr_int_copy,
-    pango_attr_int_destroy,
-    pango_attr_int_equal
-  };
+  return pango_attr_size_new_internal (size, FALSE);
+}
 
-  return pango_attr_int_new (&klass, size);
+/**
+ * pango_attr_size_absolute_new:
+ * @size: the font size, in #PANGO_SCALE<!-- -->ths of a device units.
+ * 
+ * Create a new font-size attribute in device units.
+ * 
+ * Return value: the new #PangoAttribute.
+ *
+ * Since: 1.8
+ **/
+PangoAttribute *
+pango_attr_size_new_absolute (int size)
+{
+  return pango_attr_size_new_internal (size, TRUE);
 }
 
 /**
@@ -577,6 +633,33 @@ pango_attr_underline_new (PangoUnderline underline)
   return pango_attr_int_new (&klass, (int)underline);
 }
 
+/**
+ * pango_attr_underline_color_new:
+ * @red: the red value (ranging from 0 to 65535)
+ * @green: the green value
+ * @blue: the blue value
+ * 
+ * Create a new underline color attribute. This attribute
+ * modifies the color of underlines. If not set, underlines
+ * will use the foreground color.
+ * 
+ * Return value: the new #PangoAttribute.
+ **/
+PangoAttribute *
+pango_attr_underline_color_new (guint16 red,
+				guint16 green,
+				guint16 blue)
+{
+  static const PangoAttrClass klass = {
+    PANGO_ATTR_UNDERLINE_COLOR,
+    pango_attr_color_copy,
+    pango_attr_color_destroy,
+    pango_attr_color_equal
+  };
+
+  return pango_attr_color_new (&klass, red, green, blue);
+}
+
 /**
  * pango_attr_strikethrough_new:
  * @strikethrough: %TRUE if the text should be struck-through.
@@ -598,11 +681,37 @@ pango_attr_strikethrough_new (gboolean strikethrough)
   return pango_attr_int_new (&klass, (int)strikethrough);
 }
 
+/**
+ * pango_attr_strikethrough_color_new:
+ * @red: the red value (ranging from 0 to 65535)
+ * @green: the green value
+ * @blue: the blue value
+ * 
+ * Create a new strikethrough color attribute. This attribute
+ * modifies the color of strikethrough lines. If not set, strikethrough
+ * lines will use the foreground color.
+ * 
+ * Return value: the new #PangoAttribute.
+ **/
+PangoAttribute *
+pango_attr_strikethrough_color_new (guint16 red,
+				    guint16 green,
+				    guint16 blue)
+{
+  static const PangoAttrClass klass = {
+    PANGO_ATTR_STRIKETHROUGH_COLOR,
+    pango_attr_color_copy,
+    pango_attr_color_destroy,
+    pango_attr_color_equal
+  };
+
+  return pango_attr_color_new (&klass, red, green, blue);
+}
+
 /**
  * pango_attr_rise_new:
  * @rise: the amount that the text should be displaced vertically,
- *        in 10'000ths of an em. Positive values displace the
- *        text upwards.
+ *        in Pango units. Positive values displace the text upwards.
  * 
  * Create a new baseline displacement attribute.
  * 
@@ -670,17 +779,53 @@ pango_attr_fallback_new (gboolean enable_fallback)
   return pango_attr_int_new (&klass, (int)enable_fallback);
 }
 
+/**
+ * pango_attr_letter_spacing_new:
+ * @letter_spacing: amount of extra space to add between graphemes
+ *   of the text, in Pango units.
+ * 
+ * Create a new letter-spacing attribute.
+ * 
+ * Return value: the new #PangoAttribute.
+ *
+ * Since: 1.6
+ **/
+PangoAttribute *
+pango_attr_letter_spacing_new (int letter_spacing)
+{
+  static const PangoAttrClass klass = {
+    PANGO_ATTR_LETTER_SPACING,
+    pango_attr_int_copy,
+    pango_attr_int_destroy,
+    pango_attr_int_equal
+  };
+
+  return pango_attr_int_new (&klass, letter_spacing);
+}
+
 static PangoAttribute *
 pango_attr_shape_copy (const PangoAttribute *attr)
 {
   const PangoAttrShape *shape_attr = (PangoAttrShape *)attr;
+  gpointer data;
+
+  if (shape_attr->copy_func)
+    data = shape_attr->copy_func (shape_attr->data);
+  else
+    data = shape_attr->data;
   
-  return pango_attr_shape_new (&shape_attr->ink_rect, &shape_attr->logical_rect);
+  return pango_attr_shape_new_with_data (&shape_attr->ink_rect, &shape_attr->logical_rect,
+					 data, shape_attr->copy_func, shape_attr->destroy_func);
 }
 
 static void
 pango_attr_shape_destroy (PangoAttribute *attr)
 {
+  const PangoAttrShape *shape_attr = (PangoAttrShape *)attr;
+  
+  if (shape_attr->destroy_func)
+    shape_attr->destroy_func (shape_attr->data);
+  
   g_free (attr);
 }
 
@@ -690,7 +835,7 @@ pango_attr_shape_equal (const PangoAttribute *attr1,
 {
   const PangoAttrShape *shape_attr1 = (const PangoAttrShape *)attr1;
   const PangoAttrShape *shape_attr2 = (const PangoAttrShape *)attr2;
-  
+
   return (shape_attr1->logical_rect.x == shape_attr2->logical_rect.x &&
 	  shape_attr1->logical_rect.y == shape_attr2->logical_rect.y &&
 	  shape_attr1->logical_rect.width == shape_attr2->logical_rect.width &&
@@ -698,7 +843,57 @@ pango_attr_shape_equal (const PangoAttribute *attr1,
 	  shape_attr1->ink_rect.x == shape_attr2->ink_rect.x &&
 	  shape_attr1->ink_rect.y == shape_attr2->ink_rect.y &&
 	  shape_attr1->ink_rect.width == shape_attr2->ink_rect.width &&
-	  shape_attr1->ink_rect.height == shape_attr2->ink_rect.height);
+	  shape_attr1->ink_rect.height == shape_attr2->ink_rect.height &&
+	  shape_attr1->data == shape_attr2->data);
+}
+
+/**
+ * pango_attr_shape_new_with_data:
+ * @ink_rect:     ink rectangle to assign to each character
+ * @logical_rect: logical rectangle assign to each character
+ * @data:         user data pointer
+ * @copy_func:    function to copy @data when the attribute
+ *                is copied. If %NULL, @data is simply copied
+ *                as a pointer.
+ * @destroy_func: function to free @data when the attribute
+ *                is freed, or %NULL.
+ * 
+ * Like pango_attr_shape_new(), but a user data pointer is also
+ * provided; this pointer can be accessed when rendering later
+ * rendering the glyph.
+ * 
+ * Return value: the newly created attribute
+ *
+ * Since: 1.8
+ **/
+PangoAttribute *
+pango_attr_shape_new_with_data (const PangoRectangle  *ink_rect,
+				const PangoRectangle  *logical_rect,
+				gpointer               data,
+				PangoAttrDataCopyFunc  copy_func,
+				GDestroyNotify         destroy_func)
+{
+  static const PangoAttrClass klass = {
+    PANGO_ATTR_SHAPE,
+    pango_attr_shape_copy,
+    pango_attr_shape_destroy,
+    pango_attr_shape_equal
+  };
+
+  PangoAttrShape *result;
+
+  g_return_val_if_fail (ink_rect != NULL, NULL);
+  g_return_val_if_fail (logical_rect != NULL, NULL);
+  
+  result = g_new (PangoAttrShape, 1);
+  result->attr.klass = &klass;
+  result->ink_rect = *ink_rect;
+  result->logical_rect = *logical_rect;
+  result->data = data;
+  result->copy_func = copy_func;
+  result->destroy_func =  destroy_func;
+
+  return (PangoAttribute *)result;
 }
 
 /**
@@ -717,24 +912,11 @@ PangoAttribute *
 pango_attr_shape_new (const PangoRectangle *ink_rect,
 		      const PangoRectangle *logical_rect)
 {
-  static const PangoAttrClass klass = {
-    PANGO_ATTR_SHAPE,
-    pango_attr_shape_copy,
-    pango_attr_shape_destroy,
-    pango_attr_shape_equal
-  };
-
-  PangoAttrShape *result;
-
   g_return_val_if_fail (ink_rect != NULL, NULL);
   g_return_val_if_fail (logical_rect != NULL, NULL);
-  
-  result = g_new (PangoAttrShape, 1);
-  result->attr.klass = &klass;
-  result->ink_rect = *ink_rect;
-  result->logical_rect = *logical_rect;
 
-  return (PangoAttribute *)result;
+  return pango_attr_shape_new_with_data (ink_rect, logical_rect,
+					 NULL, NULL, NULL);
 }
 
 GType
@@ -1494,7 +1676,10 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
 	  if (!(mask & PANGO_FONT_MASK_SIZE))
 	    {
 	      mask |= PANGO_FONT_MASK_SIZE;
-	      pango_font_description_set_size (desc, ((PangoAttrInt *)attr)->value);
+	      if (((PangoAttrSize *)attr)->absolute)
+		pango_font_description_set_size (desc, ((PangoAttrSize *)attr)->size);
+	      else
+		pango_font_description_set_absolute_size (desc, ((PangoAttrSize *)attr)->size);
 	    }
 	  break;
         case PANGO_ATTR_SCALE:
